SetFactory("OpenCASCADE");
If(GMSH_MAJOR_VERSION<=3 && GMSH_MINOR_VERSION<=0 && GMSH_PATCH_VERSION<=6)
  Printf("gmsh %g.%g.%g is not supported by this .geo",GMSH_MAJOR_VERSION,GMSH_MINOR_VERSION,GMSH_PATCH_VERSION);
Else
  LX = 10;
  LY = 10;
  LZ = 1;

  CX = LX/2;
  CY = LY/2;

  R = 2;
  dR = 0;

  nT = 40;
  nZ = 3;
  ndR = 20;

  PI = Acos (-1.);

  lc = 1;

  Point(201)  = { CX,    CY,  0.,     lc};

  Point(202)  = { CX+(R+dR)*Cos(1*PI/4),  CY+(R+dR)*Sin(1*PI/4), 0.,     lc};
  Point(203)  = { CX+(R+dR)*Cos(3*PI/4),  CY+(R+dR)*Sin(3*PI/4), 0.,     lc};
  Point(204)  = { CX+(R+dR)*Cos(5*PI/4),  CY+(R+dR)*Sin(5*PI/4), 0.,     lc};
  Point(205)  = { CX+(R+dR)*Cos(7*PI/4),  CY+(R+dR)*Sin(7*PI/4), 0.,     lc};

  Point(206)  = { LX,  LY, 0.,     lc};
  Point(207)  = { 0.,  LY, 0.,     lc};
  Point(208)  = { 0.,  0., 0.,     lc};
  Point(209)  = { LX,  0., 0.,     lc};

  Circle(2101) = {202, 201, 203};
  Circle(2102) = {203, 201, 204};
  Circle(2103) = {204, 201, 205};
  Circle(2104) = {205, 201, 202};

  Line(2105) = {206, 207};
  Line(2106) = {207, 208};
  Line(2107) = {208, 209};
  Line(2108) = {209, 206};

  Line(2201) = {202, 206};
  Line(2202) = {203, 207};
  Line(2203) = {204, 208};
  Line(2204) = {205, 209};

  Transfinite Line {2101, 2102, 2103, 2104, 2105, 2106, 2107, 2108} = nT Using Progression 1;
  Transfinite Line {2201, 2202, 2203, 2204} = ndR Using Progression 1;

  Line Loop(201) = {2101, 2202, -2105, -2201};
  Plane Surface(201) = {201};
  Transfinite Surface {201};
  Recombine Surface {201};

  Line Loop(202) = {2202, 2106, -2203, -2102};
  Plane Surface(202) = {202};
  Transfinite Surface {202};
  Recombine Surface {202};

  Line Loop(203) = {2103, 2204, -2107, -2203};
  Plane Surface(203) = {203};
  Transfinite Surface {203};
  Recombine Surface {203};

  Line Loop(204) = {2104, 2201, -2108, -2204};
  Plane Surface(204) = {204};
  Transfinite Surface {204};
  Recombine Surface {204};

  Extrude {0, 0, LZ}
  {
    Surface{201,202,203,204}; Layers{nZ}; Recombine;
  }

  Physical Volume("body") = {1,2,3,4};
  //+
  Physical Surface("outer part boundary") = {205,212,214,218};
  Physical Surface("load 1") = {207};
  Physical Surface("load 2") = {216};
  Physical Point("clamped xyz") = {214};
  Physical Point("clamped xy") = {208};
  Physical Point("clamped y") = {217};
EndIf
