import numpy

from katoptron.eigenvalues import *
from scipy.sparse import eye
import matplotlib.pyplot as plt
import scipy

import katoptron.utilities as u


def spectral_radii_bound(A, omega_min=0., omega_max=2., n_omega=100):
    from scipy.sparse import tril
    from scipy.sparse import triu
    from scipy.sparse import eye
    from scipy.sparse import diags
    from scipy.sparse import issparse

    omegas = np.linspace(omega_min, omega_max, n_omega)
    rhos = np.ones((n_omega, ))

    Adiag = A.diagonal()

    invD = diags(1. / Adiag, 0)

    scaled_A = invD * A

    L = -tril(scaled_A, -1)
    U = -triu(scaled_A, 1)

    n = A.shape[0]

    #n = 100

    l_n = np.zeros((n, ))
    u_n = np.zeros((n, ))

    l_2n = np.zeros((n, n - 1))
    u_2n = np.zeros((n, n - 1))

    T_l_n = np.zeros((n, ))
    T_u_n = np.zeros((n, ))
    L_abs = np.abs(L)
    U_abs = np.abs(U)

    l_n = L_abs.sum(axis=1)
    u_n = U_abs.sum(axis=1)

    T_l_n = L_abs.max(axis=1)
    T_u_n = U_abs.max(axis=1)

    if np.max(l_n) > 1:
        print('error')
        return omegas, rhos
    if np.max(T_l_n) > 1:
        print('error')
        return omegas, rhos
    '''
    for i in range(0,n):
        print i
        print n
        tmp = L.getrow(i)
        T_l_n[i] = np.max(np.abs(tmp))
        tmp = U.getrow(i)
        T_l_n[i] = np.max(np.abs(tmp))
    '''

    for i in range(0, n):
        for j in range(0, i):
            u_2n[i, j] = u_n[i] + u_n[j]
            l_2n[i, j] = l_n[i] + l_n[j]
        for j in range(i + 1, n):
            u_2n[i, j - 1] = u_n[i] + u_n[j]
            l_2n[i, j - 1] = l_n[i] + l_n[j]

    for omega_i in range(0, n_omega):
        omega = omegas[omega_i]

        tmp_1 = np.divide(2 * np.abs(1 - omega) + omega * u_2n,
                          2. - omega * l_2n)

        max_1 = np.amax(tmp_1)

        tmp_2 = np.zeros((n, ))
        for i in range(0, n):
            scalar_1 = ((np.abs(1 - omega) + omega * u_n[i]) / (1 -
                                                                      omega * l_n[i]))
            scalar_2 = ((np.abs(1 - omega)) / (1 - omega * T_u_n[i]))
            scalar_3 = np.abs(1 - omega) + omega * u_n[i]
            scalar_4 = np.max([scalar_2, scalar_3])
            tmp_2[i] = np.min([scalar_1, scalar_4])

        max_2 = np.max(tmp_2)

        rhos[i] = np.max([max_1, max_2])

    return omegas, rhos


def is_Z_matrix(A):
    from scipy.sparse import diags
    Adiag = A.diagonal()

    D = diags(Adiag, 0)

    Aoff = A - D

    max_row = Aoff.max(axis=1)

    if np.max(max_row) <= 0:
        return True
    else:
        return False


def is_M_matrix(A):
    is_Z = is_Z_matrix(A)
    if not is_Z:
        return False
    is_spd = is_SPD(A)

    return is_spd


def spectral_radii(A, omegas, n_power=10):
    from scipy.sparse import tril
    from scipy.sparse import triu
    from scipy.sparse import eye
    from scipy.sparse import diags
    from scipy.sparse import issparse

    n_omega = len(omegas)
    rhos = np.ones((n_omega, ))

    E = -tril(A, -1)
    F = -triu(A, 1)
    invD = diags(1. / A.diagonal(), 0)
    n = A.shape[0]
    I = eye(n)

    for omega_i in range(0, n_omega):
        #print omega_i
        omega = omegas[omega_i]
        if omega == 0. or omega == 2.:
            rhos[omega_i] = 1.
        else:
            N = I - omega * invD * E
            M = (1. - omega) * I + omega * invD * F
            rhos[omega_i] = np.min([gen_power_method(M, N, n=n_power), 1.])

        #print rhos

    return rhos


def main():
    comm, rank, siz = u.import_MPI()

    post_process = True

    #base_dir = '/Volumes/HD_SONY/Thesis/Juelich/GS/order_44/1/0/'

    #A_u = read_mm(base_dir+'A_00_mm.txt', -1, True, True, False)
    #A_T = read_mm(base_dir+'A_11_mm.txt', -1, True, True, True)

    # scipy.sparse.save_npz('A_T.npz',A_T)
    # scipy.sparse.save_npz('A_u.npz',A_u)

    if post_process:
        omegas = np.loadtxt('omegas.txt')
        rho_T = np.loadtxt('rho_T.txt')
        rho_u = np.loadtxt('rho_u.txt')

        omegas = np.loadtxt('results_3/omegas.txt')
        rho_T = np.loadtxt('results_3/rho_T.txt')
        rho_u_A = np.loadtxt('results_3/rho_u_A.txt')
        rho_u_B = np.loadtxt('results_3/rho_u_B.txt')

        rho_T[omegas >= 1.75] = 1.
        i_T = np.argmin(rho_T)
        i_u_A = np.argmin(rho_u_A)
        i_u_B = np.argmin(rho_u_B)

        print(omegas[i_T])
        print(omegas[i_u_A])
        print(omegas[i_u_B])

        plt.figure()
        plt.plot(omegas, rho_T)
        plt.plot(omegas, rho_u_A)
        plt.plot(omegas, rho_u_B)
        plt.show()

    else:
        print('Read data: start')

        A_T = scipy.sparse.load_npz('A_T.npz')
        A_u_A = scipy.sparse.load_npz('A_u.npz')
        A_u_B = scipy.sparse.load_npz('A_u_B.npz')

        print('Read data: end ' + str(siz))

        print('Spectral radius computation')

        n_omega = 201
        n_power = 100
        omegas = np.linspace(0., 2., n_omega)

        n_omega_per_MPI = int(np.floor(float(n_omega) / siz))
        my_first_i = n_omega_per_MPI * rank
        my_last_i = my_first_i + n_omega_per_MPI
        n_my_data = my_last_i-my_first_i

        omegas_tmp = omegas[my_first_i:my_last_i]

        rho_u_A = np.ones((n_omega, 1))
        rho_u_B = np.ones((n_omega, 1))
        rho_T = np.zeros((n_omega, 1))

        rho_u_A_tmp = spectral_radii(A_u_A, omegas_tmp, n_power=n_power)
        rho_u_B_tmp = spectral_radii(A_u_B, omegas_tmp, n_power=n_power)
        rho_T_tmp = spectral_radii(A_T, omegas_tmp, n_power=n_power)

        if rank == 0:
            np.savetxt('omegas_'+str(rank)+'.txt', omegas_tmp)
            np.savetxt('rho_T_'+str(rank)+'.txt', rho_T_tmp)
            np.savetxt('rho_u_'+str(rank)+'_A.txt', rho_u_A_tmp)
            np.savetxt('rho_u_'+str(rank)+'_B.txt', rho_u_B_tmp)

            if n_my_data > 1:
                for j in range(0, n_my_data):
                    rho_u_A[my_first_i+j] = rho_u_A_tmp[j]
                    rho_u_B[my_first_i+j] = rho_u_B_tmp[j]
                    rho_T[my_first_i+j] = rho_T_tmp[j]
            else:
                rho_u_A[my_first_i] = rho_u_A_tmp
                rho_u_B[my_first_i] = rho_u_B_tmp
                rho_T[my_first_i] = rho_T_tmp

            for i in range(1, siz):
                first_i = n_omega_per_MPI * i
                last_i = first_i + n_omega_per_MPI

                n_data = last_i-first_i

                data = comm.recv(source=i, tag=11)

                if n_data > 1:
                    for j in range(0, n_data):
                        rho_u_A[first_i+j] = data[j]
                else:
                    rho_u_A[first_i] = data

                data = comm.recv(source=i, tag=11)

                if n_data > 1:
                    for j in range(0, n_data):
                        rho_u_B[first_i+j] = data[j]
                else:
                    rho_u_B[first_i] = data

                data = comm.recv(source=i, tag=11)

                if n_data > 1:
                    for j in range(0, n_data):
                        rho_T[first_i+j] = data[j]
                else:
                    rho_T[first_i] = data
        else:
            np.savetxt('omegas_'+str(rank)+'.txt', omegas_tmp)
            np.savetxt('rho_T_'+str(rank)+'.txt', rho_T_tmp)
            np.savetxt('rho_u_'+str(rank)+'_A.txt', rho_u_A_tmp)
            np.savetxt('rho_u_'+str(rank)+'_B.txt', rho_u_B_tmp)
            comm.send(rho_u_A_tmp, dest=0, tag=11)
            comm.send(rho_u_B_tmp, dest=0, tag=11)
            comm.send(rho_T_tmp, dest=0, tag=11)

        if rank == 0:
            print(omegas)
            print(rho_T)
            print(rho_u_A)
            print(rho_u_B)

            np.savetxt('omegas.txt', omegas)
            np.savetxt('rho_T.txt', rho_T)
            np.savetxt('rho_u_A.txt', rho_u_A)
            np.savetxt('rho_u_B.txt', rho_u_B)


if __name__ == "__main__":
    main()
