#!/usr/bin/env python3
# -*- coding: utf-8; -*-

import katoptron as m
import tbox
import tbox.gmsh as gmsh
import katoptron.utilities as u
import numpy as np
import shlex
import subprocess
import os
import tarfile
import glob

from katoptron.Juelich.UQ.halton import *
from katoptron.Juelich.UQ.copulas import *


def evaluate_one_ensemble(ensemble_size,
                          msh,
                          comm,
                          file_dir,
                          randomVariable_1,
                          randomVariable_2,
                          randomVariable_3,
                          scale_u,
                          scale_T,
                          option,
                          baking=False):

    from fwk.wutils import parseargs
    args = parseargs()

    from PyTrilinos import Teuchos
    import numpy as np

    small_disk_footprint = False
    use_block = True

    comm, rank, siz = m.utilities.import_MPI()

    solverList = Teuchos.ParameterList()
    solverList['type'] = "BlockGmres"

    solverList['Maximum Iterations'] = 500
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList['Use preconditioner'] = True

    solverList['Print Teuchos timers'] = True
    solverList['Write txt files'] = True
    if small_disk_footprint:
        solverList['Write vtk files'] = False

    solverList['convert MueLu xml file'] = True
    solverList["use xml file"] = True

    solverList['Ensemble Convergence Tolerance'] = 10**(-8)
    solverList[
        "MueLu xml file name"] = file_dir + '/../smoothers/heat_no_contact_mtgs.xml'

    solverList["Scaled Block 0"] = True
    pbl = m.Problem(msh, comm)


def main():
    comm, rank, size = m.utilities.import_MPI()

    from PyTrilinos import Teuchos
    import numpy as np

    geo_name = 'contact_mirror_assembly_mm.geo'
    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    msh, mesh_name, partitioned_mesh_name = u.read_mesh_MPI(
        geo_name, file_dir+'/../mesh', work_dir, comm, rank, size)

    solverList = Teuchos.ParameterList()
    solverList['type'] = "BlockGmres"

    solverList['Maximum Iterations'] = 500
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList['Use preconditioner'] = True

    solverList['Print Teuchos timers'] = True
    solverList['Write txt files'] = True

    solverList['convert MueLu xml file'] = True
    solverList["use xml file"] = True

    solverList['Ensemble Convergence Tolerance'] = 10**(-8)
    solverList[
        "MueLu xml file name"] = file_dir + '/../smoothers/heat_no_contact_mtgs.xml'

    solverList["Scaled Block 0"] = True

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    pbl = m.Problem(msh, comm)

    ensemble_size = 1

    scale_u = 1000.
    scale_T = 1.

    scale_E = 1. / scale_u**2
    scale_k = (scale_T / scale_u)
    scale_hsource = 1. / scale_u**3
    scale_hflux = 1. / scale_u**2
    scale_d = scale_T

    # 379 000 MPa = 379 GPa = 379E9 Pa
    E_rhodium = 379E9 * np.ones(ensemble_size) * scale_E
    nu_rhodium = 0.26 * np.ones(ensemble_size)
    # 0.150 kW/(m K) = 150 W/(m K)
    k_rhodium = 150 * scale_k
    d_rhodium = 0.781E-05 * scale_d

    E_WCu = 280E9 * np.ones(ensemble_size) * scale_E
    nu_WCu = 0.298 * np.ones(ensemble_size)
    k_WCu = 180 * scale_k
    d_WCu = 0.88E-05 * scale_d

    E_SS = 200E9 * np.ones(ensemble_size) * scale_E
    nu_SS = 0.3 * np.ones(ensemble_size)
    k_SS = 15.3 * scale_k
    d_SS = 1.57E-05 * scale_d

    E_CuCrZr = 118E9 * np.ones(ensemble_size) * scale_E
    nu_CuCrZr = 0.33 * np.ones(ensemble_size)
    k_CuCrZr = 345 * np.ones(ensemble_size) * scale_k
    d_CuCrZr = 1.8E-05 * np.ones(ensemble_size) * scale_d

    E_AlN = 320E9 * np.ones(ensemble_size) * scale_E
    nu_AlN = 0.24 * np.ones(ensemble_size)
    k_AlN = 180 * np.ones(ensemble_size) * scale_k
    d_AlN = 0.48E-05 * scale_d

    E_I718 = 183E9 * np.ones(ensemble_size) * scale_E
    nu_I718 = 0.31 * np.ones(ensemble_size)
    k_I718 = 15.8 * scale_k
    d_I718 = 1.38E-05 * scale_d

    T_ref = 22.

    m.Medium(pbl, "Mirror", "rhodium", E_rhodium, nu_rhodium, k_rhodium,
             d_rhodium)
    m.Medium(pbl, "Substrate", "WCu", E_WCu, nu_WCu, k_WCu, d_WCu)

    m.Medium(pbl, "Holder", "CuCrZr", E_CuCrZr, nu_CuCrZr, k_CuCrZr, d_CuCrZr)

    m.Medium(pbl, "Washer 1", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Washer 2", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Washer 3", "I718", E_I718, nu_I718, k_I718, d_I718)

    m.Medium(pbl, "Shaft 1", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Shaft 2", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Shaft 3", "I718", E_I718, nu_I718, k_I718, d_I718)

    m.Medium(pbl, "Nut 1", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Nut 2", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Nut 3", "I718", E_I718, nu_I718, k_I718, d_I718)

    m.Medium(pbl, "Spacer 1 part 1", "AlN", E_AlN, nu_AlN,
             180. * scale_k, d_AlN)
    m.Medium(pbl, "Spacer 2 part 1", "AlN", E_AlN, nu_AlN,
             180. * scale_k, d_AlN)
    m.Medium(pbl, "Spacer 3 part 1", "AlN", E_AlN, nu_AlN,
             180. * scale_k, d_AlN)

    m.Medium(pbl, "Spacer 1 part 2", "WCu", E_WCu, nu_WCu, k_WCu, d_WCu)
    m.Medium(pbl, "Spacer 2 part 2", "WCu", E_WCu, nu_WCu, k_WCu, d_WCu)
    m.Medium(pbl, "Spacer 3 part 2", "WCu", E_WCu, nu_WCu, k_WCu, d_WCu)

    m.Dirichlet(pbl, "Holder x-y-z clamping point", "Clamped", 1, 0., 1, 0., 1,
                0., 0, 0., ensemble_size)
    m.Dirichlet(pbl, "Holder z clamping point", "Clamped", 0, 0., 0, 0., 1, 0.,
                0, 0., ensemble_size)
    m.Dirichlet(pbl, "Holder z-y clamping point", "Clamped", 0, 0., 1, 0., 1,
                0., 0, 0., ensemble_size)

    m.Dirichlet(pbl, "Cooling channels", "Clamped", 0, 0., 0, 0., 0, 0., 1,
                ((70. - T_ref) / scale_T), ensemble_size)

    m.Dirichlet(pbl, "Mirror surface", "Clamped", 1, 0., 1, 0., 1, -10., 1,
                ((70. - T_ref) / scale_T), ensemble_size)
    '''
    zero = np.zeros(ensemble_size)
    hf = 20. / 0.0121 * np.ones(
        ensemble_size) * scale_hflux  # 20 W / (0.0121 m^2)

    m.Neumann(pbl, "Mirror surface", "heat fluxes", 0, zero, 0, zero, 0, zero,
                1, hf, ensemble_size)

    m.Source(pbl, "Mirror", "ihg", 20E6 * scale_hsource)
    m.Source(pbl, "Substrate", "ihg", 2.45E6 * scale_hsource)

    if material_option(option) == 1:
        m.Source(pbl, "Holder", "ihg", 0.7E6 * scale_hsource)
    elif material_option(option) == 2:
        m.Source(pbl, "Holder", "ihg", 0.6E6 * scale_hsource)

    m.Source(pbl, "Spacer 1 part 1", "ihg", 0.3E6 * scale_hsource)
    m.Source(pbl, "Spacer 2 part 1", "ihg", 0.3E6 * scale_hsource)
    m.Source(pbl, "Spacer 3 part 1", "ihg", 0.3E6 * scale_hsource)

    m.Source(pbl, "Spacer 1 part 2", "ihg", 2.45E6 * scale_hsource)
    m.Source(pbl, "Spacer 2 part 2", "ihg", 2.45E6 * scale_hsource)
    m.Source(pbl, "Spacer 3 part 2", "ihg", 2.45E6 * scale_hsource)

    m.Source(pbl, "Shaft 1", "ihg", 0.8E6 * scale_hsource)
    m.Source(pbl, "Shaft 2", "ihg", 0.8E6 * scale_hsource)
    m.Source(pbl, "Shaft 3", "ihg", 0.8E6 * scale_hsource)

    m.Source(pbl, "Nut 1", "ihg", 0.8E6 * scale_hsource)
    m.Source(pbl, "Nut 2", "ihg", 0.8E6 * scale_hsource)
    m.Source(pbl, "Nut 3", "ihg", 0.8E6 * scale_hsource)

    m.Source(pbl, "Washer 1", "ihg", 0.8E6 * scale_hsource)
    m.Source(pbl, "Washer 2", "ihg", 0.8E6 * scale_hsource)
    m.Source(pbl, "Washer 3", "ihg", 0.8E6 * scale_hsource)
    '''

    nThreads = u.Initialize_Kokkos()
    slv = m.IterativeSolver(pbl, solverList, 4, ensemble_size)

    slv.start()
    u.Finalize_Kokkos()


if __name__ == "__main__":
    main()
